/* -*- mode: C++; c-basic-offset: 4; indent-tabs-mode: nil; -*- */
/*
 * SPDX-License-Identifier: MIT OR LGPL-2.0-or-later
 *
 * Copyright (c) 2020 Evan Welsh <noreply@evanwelsh.com>
 */

#ifndef MODULES_PRINT_H_
#define MODULES_PRINT_H_

#include <config.h>

#include <js/TypeDecls.h>

#include "gjs/macros.h"

GJS_JSAPI_RETURN_CONVENTION
bool gjs_define_print_stuff(JSContext* context, JS::MutableHandleObject module);

#endif  // MODULES_PRINT_H_
